/**
* Copyright 
* Jens GUSTEDT (INRIA) jens.gustedt@loria.fr
* Emmanuel JEANNOT (INRIA) emmanuel.jeannot@loria.fr
* Marc THIERRY (INRIA, 2004)
* Louis-Claude CANON (INRIA, 2005)
* Olivier DUBUISSON (INRIA 2007) olivier.dubuisson@loria.fr
*
* This software is a computer program whose purpose is to define and 
* control the heterogeneity of a given platform by degrading CPU, network 
* or memory capabilities of each node composing this platform.
* The degradation is done remotely, without restarting the hardware. 
* The control is fine, reproducible and independent 
* (one may degrades CPU without modifying the network bandwidth).
* 
* This software is governed by the CeCILL license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
* 
* As a counterpart to the access to the source code and  rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
* 
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
* 
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL license and that you accept its terms.
*
*
* Some part of this progam were taken from the procstat program of   
* Ivan Tikhonov, kefeer@brokestream.com
*
* http://brokestream.com/procstat.html 
*

*/

#include <asm/param.h>
#include <ctype.h>
#include <errno.h>
#include <sched.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <linux/limits.h>
#include <sys/times.h>

#define UDIFF_SJ(a,b) (a*1e+6)-(b*1e+6)/HZ

pid_t pid;
long long time_tot = 0, work_time_tot = 1;

void quit (int sig);


typedef long long int num;

num mypid;
char tcomm[PATH_MAX];
char state;

num ppid;
num pgid;
num sid;
num tty_nr;
num tty_pgrp;

num flags;
num min_flt;
num cmin_flt;
num maj_flt;
num cmaj_flt;
num utime;
num stimev;

num cutime;
num cstime;
num priority;
num nicev;
num num_threads;
num it_real_value;

unsigned long long start_time;

double wall_time;
double cpu_time;

long tickspersec;

int gettimesinceboot() {
    FILE *procuptime;
    int sec, ssec;

    procuptime = fopen("/proc/uptime", "r");
    fscanf(procuptime, "%d.%ds", &sec, &ssec);
    fclose(procuptime);
    return (sec*tickspersec)+ssec;
}

double percentage(int pid){
    int sinceboot = gettimesinceboot();
    int running;
    double percentage;
    char filename[100];
    FILE *infile;
    char input[1000];

    sprintf(filename,"/proc/%d/stat",pid);

    if (infile = fopen(filename, "r")){
        fgets(input,1000,infile);
        fclose(infile);
        sscanf(input,"%lld %s %c %lld %lld %lld %lld %lld %lld %lld"
            " %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %llu",
            &mypid,tcomm,&state,&ppid,&pgid,
            &sid,&tty_nr,&tty_pgrp,&flags,&min_flt,
            &cmin_flt,&maj_flt,&cmaj_flt,&utime,&stimev,
            &cutime,&cstime,&priority,&nicev,&num_threads,
            &it_real_value,&start_time);
        running= sinceboot - start_time;
        wall_time= (running*1.0) / tickspersec;
        cpu_time = ((double)(utime+stimev))/ tickspersec;
        percentage=cpu_time/wall_time;
        //printf("\r Wall: %.2f CPU: %.2f percent : %.2f",wall_time,cpu_time,percentage);
        return percentage;
    } else {
        return -1;
    }
}


int
main (int argc, char **argv)
{
    float percent;
    double cur_percent;
    int sleep_time=1;

    if (argc != 3) {
        printf ("Usage: %s percent pid\n", argv[0]);
        exit (-1);
    }

    percent = atof (argv[1]);
    pid = atoi (argv[2]);
    if (percent > 1. || percent <= 0.) {
        perror ("Bad percentage");
        exit (-1);
    }

    if (percent != 1.)
    {
        int flag = 1;
        char filename[80], state;
        extern int errno;
        struct sched_param min;

        tickspersec = sysconf(_SC_CLK_TCK);  

        min.sched_priority = sched_get_priority_min (SCHED_FIFO);
        if (0 && sched_setscheduler (0, SCHED_FIFO, &min) == -1) {
            perror ("Was not able to acquire RT priority (however this is ok)");
        }

        /* use the normal procedure for quiting */
        signal (SIGHUP, quit);
        signal (SIGINT, quit);
        signal (SIGTERM, quit);

        sprintf (filename, "/proc/%d/stat", pid);
        while (1) {
            cur_percent=percentage(pid);
            
            if (cur_percent < 0){
                if (wall_time)
                    printf
                    ("STAT for PID = %d  =>  CPU: %.2fs, Wall: %.2fs, Percent: %.2f\n",
                        pid, cpu_time, wall_time,
                        cpu_time * 100. / (double) wall_time);
                exit (-1);    
            }

            if (cur_percent > percent)
            {
                if (flag)
                {
                    if (state == 'T' || state == 'S')
                        continue;
                    if (!kill (pid, SIGSTOP) && errno == ESRCH)
                    {
                        printf ("Controlled program is finished\n");
                        quit (0);
                    }
                    flag = 0;
                }
            }
            else if (!flag) {
                if (!kill (pid, SIGCONT) && errno == ESRCH)
                {
                    printf ("Controlled program is finished\n");
                    quit (0);
                }
                flag = 1;
            }
            /* Here: some usleep, depending on the granularity
             * and the amount of cpu avalaible (ie, it takes many
             * cpu without at least one usleep */

            usleep (sleep_time);
            /*be every reactive at the begining, can sleep longer if the process has run for a while*/
            if(sleep_time<100)
                sleep_time++;
        }
    }
    return 0;
}

void
quit (int sig)
{
    /* if the controled process was stopped and this program is stop,
     * we continue the normal execution of the process */
    kill (pid, SIGCONT);
    if (wall_time)
        printf
        ("STAT for PID = %d  =>  CPU: %.2fs, Wall: %.2fs, Percent: %.2f\n",
         pid, cpu_time, wall_time,
         cpu_time * 100. / (double) wall_time);
    exit (-1);
}
