# -*- coding: utf-8 -*-
# takes care of cpufreq stuff
# it is said that /proc/acpi/* is deprecated 
# ee, who cares

import os.path
import os
import rollback
from logging import log
import procinfo

def get_scaled():
	'''give procs with changing frequency'''
	path = '/proc/acpi/processor'
	cpus = os.listdir(path)
	throttled = []
	frequed   = []
	for cpu in cpus:
		f = open('%s/%s/info' % (path, cpu), 'r')
		lines = f.readlines()
		f.close()
		throttle = freq = False
		procid = None
		for line in lines:
			name, value = line.split(':')
			name, value = name.strip(), value.strip()
			if name == 'processor id':
				procid = int(value)
			elif name == 'throttling control' and value == 'yes': # thermal throttling
				throttle = True
			elif name == 'power management' and value == 'yes': # cpufreq?
				freq = True
		if throttle:
			throttled.append(procid)
		if freq:
			frequed.append(procid)
	return (throttled, frequed)

def get_throttled():
	return get_scaled()[0]

def get_cpufreq():
	scaled = []
	path = '/sys/devices/system/cpu'
	procs = procinfo.getinfo()['procs']
	for cpuid in procs:
		cp = '%s/cpu%d/cpufreq' % (path, cpuid)
		if os.path.exists(cp):
			scaled.append(cpuid)
	return scaled

def get_path(cpu, suffix):
	return '/sys/devices/system/cpu/cpu%d/cpufreq/%s' % (cpu, suffix)

def get_governor(cpu):
	return get_path(cpu, 'scaling_governor')

def freq_rollback(pid, governor):
	log('Resetting governor on cpu %d...' % pid)
	path = get_governor(pid)
	f = open(path, 'w')
	f.write(governor)
	f.close()

def disable_cpufreq():
	'''The idea is to set the governor to performance'''
	procs = get_cpufreq()  # cores with cpu freq. scaling
	if len(procs) == 0:
		log('Not needed.')
	for pid in procs:
		path = get_governor(pid)
		f = open(path, 'r')
		governor = f.readline().strip()
		f.close()
		f = open(path, 'w')
		f.write('performance')
		f.close()
		rollback.register(freq_rollback, [ pid, governor ]) # to reset CPU conf.
	return None
