# -*- coding: utf-8 -*-
# linux scheduler routines

import ctypes
import os

# this is from <bits/sched.h>
SCHED_OTHER = 0
SCHED_FIFO = 1
SCHED_RR = 2

libc = ctypes.cdll.LoadLibrary('libc.so.6')

# this is from: http://stackoverflow.com/questions/661017/access-to-errno-from-python
get_errno_loc = libc.__errno_location
get_errno_loc.restype = ctypes.POINTER(ctypes.c_int)

# access to errno is REALLY tacky
def errcheck(ret, func, args):
    if ret == -1:
        e = get_errno_loc()[0]
        raise OSError(os.strerror(e))
    return ret

sched_getscheduler = libc.sched_getscheduler
sched_getscheduler.argtypes = [ctypes.c_int]
sched_getscheduler.errcheck = errcheck

# this is prepared basing on VERY strong assumptions
sched_setscheduler = libc.sched_setscheduler
sched_setscheduler.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_voidp]
sched_setscheduler.errcheck = errcheck

def getscheduler(pid):
	return sched_getscheduler(pid)

def setscheduler(pid, policy, param):
	i = ctypes.c_int(param)
	return sched_setscheduler(pid, policy, ctypes.byref(i))

if __name__ == '__main__':
	print getscheduler(0)
	print setscheduler(0, SCHED_FIFO, 1)