# -*- coding: utf-8 -*-
# Common communication routines for distest framework.

import pickle
import struct
import time
import select

PORT = 10310

def send_obj(s, obj):
	raw = pickle.dumps(obj)
	n = len(raw)
	pack = struct.pack('!i', n)
	msg = pack + raw
	s.sendall(msg)

class Error(Exception):
	def __init__(self, msg, cl = None):
		Exception.__init__(self, msg)
		self.client = cl

def readall(s, n, timeout):
	poll = select.poll()
	poll.register(s, select.POLLIN)
	r = ''
	start = time.time()
	while time.time() - start < timeout and len(r) != n:
		l = poll.poll(100)
		if len(l) == 0:
			continue
		part = s.recv(n - len(r))
		if not part: # disconnected
			raise Error('disconnected')
		r += part
	if len(r) != n:
		raise Error('timeout')
	return r

def recv_obj(s, timeout = 5):
	start = time.time()
	size = struct.calcsize('!i')
	header = readall(s, size, timeout)
	header = struct.unpack('!i', header)[0]
	passed = time.time() - start
	raw = readall(s, header, timeout - passed)
	return pickle.loads(raw)
