#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Distest server
# Reads specified test from a given file and starts to serve on TCP port.
# You can then use client.py to connect to it and perform the tests.
# Results are saved to a specified directory for later processing.

import sys
import os
import shutil
import random
import platform
from getopt import getopt, GetoptError
from datetime import datetime
from os.path import dirname, abspath, join
from tests import *
from server import DistestServer
from server import TIMEOUT, PORT
from subprocess import Popen, PIPE

def Sout(path):
	f = open('/dev/null', 'w')
	p = Popen([ path ], shell = True, stdout=PIPE, stderr=f)
	out = p.communicate()[0]
	f.close()
	return out

def bashlist(path, f = lambda x: x):
	return map(f, Sout(path).split())

def get_revision():
	# output is like 'commit c85cc72b7b20781d873a5dbed8b3a6610d53903d'
	mydir = dirname(abspath(sys.argv[0]))
	try:
		return bashlist('cd %s; git log | head -n 1' % mydir)[1] 
	except IndexError: # no git?
		return 'UNKNOWN REVISION'

try:
	opts, args = getopt(sys.argv[1:], 'p:')
	opts = dict(opts)
	if len(args) != 2:
		raise GetoptError('Syntax: python distest.py <test-spec> <output-dir>')
	proxy = opts.get('-p', None)
except GetoptError, err:
	print str(err)
	sys.exit(1)

filename = args[0]
output = args[1]
logfile = join(output, '!log')

if os.path.exists(output):
	shutil.rmtree(output)
os.makedirs(output)

suite = { 'timeout' : TIMEOUT }
execfile(filename, suite)

try:
	timeout = suite['timeout']
	assert 'tests' in suite, 'No tests defined.'
	testsuite = list(suite['tests']())
	assert all(map(lambda t: isinstance(t, Test), testsuite)), 'Not all tests are properly defined!'
except AssertionError, e:
	print 'No proper tests defined:', e
	sys.exit(1)
except NameError:
	print 'No tests defined?'
	sys.exit(1)
except TypeError:
	print 'Type error?'
	sys.exit(1)

# tests are provided
# let's shuffle them
random.shuffle(testsuite)

def tee(filename): # tee
	_fp = open(filename, 'w')
	def _log(msg):
		now = datetime.now()
		prefix = '[ %02d:%02d:%02d.%06d ] ' % ( now.hour, now.minute, now.second, now.microsecond )
		msg = prefix + msg
		print msg
		_fp.write(msg + '\n')
	return _log

logger = tee(logfile)
now = datetime.now()
date = now.strftime('Today is %d %b %Y.')

logger(date)
logger('Distest server is starting...')
logger('Cmdline is %s' % ' '.join(sys.argv))
logger('Using Python %s' % platform.python_version())
logger('Platform is %s' % platform.platform())
logger('Repository commit is %s' % get_revision())
logger('There are %d tests to perform.' % (len(testsuite),) )
logger('Setting timeout value to %d.' % (timeout,) )

if proxy:  # ssh host to tunnel connection
	spec = ':%d:localhost:%d' % (PORT, PORT)
	args = [ 'ssh', '-N', '-R', spec, proxy]
	logger('Spawning SSH proxy: "%s"...' % (' '.join(args),))
	proxy = Popen(args, stdout = PIPE, stderr = PIPE)

server = DistestServer(testsuite, output, log = logger)
server.set_timeout(timeout)

try:
	server.serve()
except KeyboardInterrupt:
	logger('Keyboard interrupt handled.')
	logger('Performed %d tests from total %d.' % (len(server.done), server.n))
	logger('See you next time!')

if proxy:
	logger('Stopping proxy...')
	proxy.terminate()
	proxy.wait()