# -*- coding: utf-8 -*-
# Simple routines to save many files in one file.
# Basically it takes a Python dictionary and encodes it in the following format:
# dictkey1:
#  indented (one-space) value
#  second line
# end
# dictkey2:
#  again indented value
# end

def encode(s):
	'''pad all lines with one space and one space at the end'''
	lines = s.split('\n')
	lines = [ ' ' + line for line in lines ]
	return '\n'.join(lines) + '\n'

def decode(s):
	'''inverse transform'''
	assert s[-1] == '\n'
	lines = s.split('\n')
	lines.pop()  # remove additional line
	assert all([ line.startswith(' ') for line in lines ])
	lines = [ line[1:] for line in lines ]
	return '\n'.join(lines)

def save(f, d):
	fs = sorted(d.keys())
	for field in fs:
		f.write('%s:\n' % field)
		f.write(encode(d[field]))
		f.write('end\n')
	pass

def load(f):
	d = {}
	cont = ''
	name = None
	for line in f:
		if line == 'end\n':
			d[name] = decode(cont)
			name = None
			cont = ''
		elif not line.startswith(' '):
			name = line.strip()[:-1]
			cont = ''
		else:
			cont += line
	return d

def loadfile(fname):
	f = open(fname, 'r')
	try:
		return load(f)
	finally:
		f.close()

if __name__ == '__main__':
	a = '''cze
jestem
tomek'''
	b = '''a ja 
	jestem
	ziomek'''
	import StringIO
	f = StringIO.StringIO()
	d = { 'aha' : a, 'bebe' : b }
	save(f, d)
	f = StringIO.StringIO(f.getvalue())
	assert load(f) == d