# -*- coding: utf-8 -*-
# statistics utilities

import math
import mpmath as mp

mp.dps = 20

def mean(L):
	n = len(L)
	return sum(L) / float(n)

def var(L):
	n = len(L)
	m = mean(L)
	return sum(map(lambda x: (x - m)**2, L)) / float(n - 1)

def stddev(L):
	return math.sqrt(var(L))

# t-student

def tpdfunc(v):
	# wikipedia
	c = mp.gamma((v + 1) * 0.5) / (mp.sqrt(v * mp.pi) * mp.gamma(v * 0.5))
	def f(t):
		return c * ( (1 + t*t / v) ** (-(v + 1) * 0.5))
	return f

def tcdfunc(v):
	pdf = tpdfunc(v)
	def f(t):
		return mp.quad(pdf, [0, t]) + 0.5
	return f

_cache = {}
def tdist(v, percent):
	pair = (v, percent)
	if pair in _cache:
		return _cache[pair]
	t = tcdfunc(v)
	v = mp.findroot(lambda x: t(x) - percent, [0,5], solver='anderson')
	_cache[pair] = v
	return v

def tinterval(percent, L):
	n = len(L)
	m = mean(L)
	s = stddev(L)
	c = tdist(n - 1, 0.5 + percent * 0.5)
	d = c * s / mp.sqrt(n)
	return ( float(m - d), float(m + d), float(d) )
