# -*- coding: utf-8 -*-
# This file defines Env and Test classes.
# The first one specifies environment where the test is supposed to run.
# This includes for example scheduler's parameters and required cores' speeds 
# (but may include also sth else in the future).

from os.path import join

def product2(dims, tab = [], pos = 0):
	if len(dims) - 1 == pos:
		for el in dims[-1]:
			tab.append(el)
			yield tuple(tab)
			tab.pop()
	else:
		for el in dims[pos]:
			tab.append(el)
			for prod in product2(dims, tab, pos + 1):
				yield prod
			tab.pop()

def product(*dims):
	'''Returns a cartesian product of all lists given as arguments'''
	return product2(dims)

# this is to avoid duplication of the code

from runner import all_benchmarks as BENCHES
from runner import all_engines as ENGINES

BENCHES = list(BENCHES)
ENGINES = list(ENGINES)

RELAX = {
	-1 : 'DEFAULT',
	0 : 'NONE',
	1 : 'SIBLINGS',
	2 : 'CORES',
	3 : 'CPUS',
	4 : 'NODES',
	5 : 'ALL',
	}

RELAXES = list(RELAX)

PARAPIDE = {
	'cores': 8,
	'speeds' : [ 2933000,2800000,2667000,2533000,2400000,2267000,2133000,2000000,1867000,1733000,1600000 ],
	}

GRIFFON = {
	'cores': 8,
	'speeds' : [ 2500000, 2000000 ],
}

TOMA = {
	'cores' : 1,
	'speeds' : [ 1867000,1600000,1333000,1067000,800000 ],
	}

CHTI = {
	'cores' : 2,
	'speeds' : [ 2600000, 2400000, 2200000, 2000000, 1800000, 1000000 ],
	}

CLUSTERS = {
	'parapide' : PARAPIDE,  # Grid5000 cluster at Rennes
	'toma' : TOMA,          # my old laptop :)
	'griffon' : GRIFFON,    # Grid5000 cluster at Nancy
	'chti' : CHTI,
	}

class Test(dict):
	def __init__(self, **kwds):
		dict.__init__(self)
		self.unique = ''
		assert 'cluster' in kwds, 'Test cluster must be specified'
		cluster = kwds['cluster']
		assert cluster in CLUSTERS, 'Unknown cluster: %s' % cluster
		self.speeds = CLUSTERS[cluster]['speeds']
		self.cores = CLUSTERS[cluster]['cores']
		for key in kwds:
			self[key] = kwds[key]
		self.cluster = cluster
	
	def map_freq(self, freq):
		if isinstance(freq, str):
			if freq[0] in ['-', '+']:
				freq = self.speeds[int(freq)]
			else:
				raise AssertionError('unknown frequency specifier: %s' % freq)
		elif isinstance(freq, int):
			assert (freq in self.speeds) or (freq == 0), 'unknown frequency for cluster %s: %s' % (self.cluster, freq)
		else:
			raise AssertionError('unknown frequency specifier: %s' % freq)
		return freq
	
	def interpret_freq(self, value):
		if isinstance(value, list):
			value = [ self.map_freq(x) for x in value ]
			spec = len(value)
			value = value + ([ 0 ] * (self.cores - spec))
		elif isinstance(value, str):
			freq = self.map_freq(value)
			value = [ freq ] * self.cores
		elif isinstance(value, int):
			value = [ value ] * self.cores
		else:
			raise AssertionError('Unknown frequency specifier: %s' % value)
		return value

	def __setitem__(self, name, value):
		self.unique += '-[%s=%s]' % (name, value)
		if name == 'speeds':
			value = self.interpret_freq(value)
		elif name == 'bench':
			assert value in BENCHES, 'No such test: %s' % (test,)
		elif name == 'engine':
			assert value in ENGINES, 'No such engine: %s' % (eng,)
		elif name in [ 'latency', 'granularity' ]:
			assert isinstance(value, int), 'Latency and granularity must be an integer'
		dict.__setitem__(self, name, value)
	
	def default_output(self):
		self.set_unique()
	
	def set_output(self, fmt):
		self['output'] = fmt % self
	
	def set_unique(self):
		self['output'] = self.unique[1:]
	
def get_speeds(cluster):
	return CLUSTERS[cluster]['speeds']

def get_cores(cluster):
	return CLUSTERS[cluster]['cores']

def get_cluster(cluster):
	return get_cores(cluster), get_speeds(cluster)

if __name__ == '__main__':
	pass
