# -*- coding: utf-8 -*-

from tests import *
from postprocess import *

timeout = 60 * 5   # five minutes!

cores, speeds = get_cluster('parapide')

latencies  = [ 10**5, 10**6, 10**7, 10**8, 10**9 ]
dims = [ latencies, speeds, ['linpack', 'threads'], range(10) ]

def tests():
	'''This generator defines test to perform'''
	for lat, speed, bench, id in product(*dims):
		t = Test(name = str(speed), cluster = 'parapide', bench = bench, engine = 'fracas', id = id)
		t['latency'] = lat
		t['granularity'] = lat
		t['speeds'] = speed
		t.default_output()
		yield t

scalefreq = lambda x: x * 0.000001
def postprocess():
	def conv_lat(lat):
		lat /= 1000000.
		if lat < 1.:
			lat = '%.1f' % lat
		else:
			lat = '%d' % lat
		return '%s ms' % (lat,)
	def flat(row):
		# convert to GigaHertz and GigaFLOPS
		if row.bench == 'linpack':
			row = row.replace(result = row.result * 1000)
		return row.replace(speeds = scalefreq(row.speeds[0]), latency = conv_lat(row.latency))
	res = get_results()
	res = res.select(['bench', 'latency', 'id', 'result', 'speeds'], f = flat)
	res = average_results(res, count = 10)
	for bench, series in group_series(res, group = 'bench', series = 'latency',
					x = 'speeds', y = 'result', yerr = 'error'):
		pdf = get_output('latency-' + bench + '.pdf')
		eps = get_output('latency-' + bench + '.eps')
		txt = get_output('latency-' + bench + '.txt')
		margs = { 'left' : 0.12, 'right' : 0.96 }
		kwds = { 'title':None, 'xlabel':'CPU Frequency [GHz]', 'ylabel':bench, 'margins' : margs }
		save_series(series, pdf, **kwds)
		save_series(series, eps, **kwds)
		dump_series(series, txt)
