# -*- coding: utf-8 -*-

from tests import *
from postprocess import *

scalefreq = lambda x: x * 0.000001
margin = 0.1

def compute_xlimits(res):
	ranges = {}
	for row in res.select(['inputdir', 'cluster']).unique().iterrows():
		rang = CLUSTERS[row.cluster]['speeds']
		rang = map(scalefreq, rang)
		span = rang[0] - rang[-1]; marg = span * margin
		ranges[row.inputdir] = (rang[-1] - marg, rang[0] + marg)
	return ranges

def dirs_to_clusters(res):
	d = {}
	for row in res.select(['inputdir', 'cluster']).unique().iterrows():
		d[row.inputdir] = row.cluster
	return d

def latex_figures(fname, graphs):
	f = open(fname, 'w')
	graphs.sort()
	count = 0
	for bench, cluster, graph in graphs:
		if count % 3 == 0:
			f.write('\\clearpage\n\n')
		case = '(all cores emulated)' if graph.endswith('all') else '(a single core emulated)'
		s = r'''\begin{figure}[!t]
		\centering
		\includegraphics[width=0.5\textwidth]{graphs/%s}
		\caption{%s benchmark on the %s cluster %s}
		\label{fig:%s}
		\end{figure}%s''' % (graph, human_bench(bench), cluster.capitalize(), case, graph, '\n\n')
		f.write(s)
		count += 1
	f.close()

def postprocess():
	def flat(row):
		# convert to GigaHertz and GigaFLOPS
		if row.bench == 'linpack':
			row = row.replace(result = row.result * 1000)
		return row.replace(speeds = scalefreq(row.speeds[0]))
	res = get_results()
	xlims = compute_xlimits(res)
	d2c = dirs_to_clusters(res)
	res = res.select(['inputdir', 'bench', 'engine', 'id', 'result', 'speeds'], f = flat)   # take these columns
	res = res.filter(lambda x: x.engine not in ['cpulim2', 'cpulim3'])   # remove engine not described in paper
	res = average_results(res, count = 10)   # compute default confidence intervals
	ylims = compute_ylimits(res, col = 'bench', margin = margin)
	graphs = []
	for idir, rest in res.groupone('inputdir').iteritems():
		for bench, series in group_series(rest, group = 'bench', series = 'engine',
					x = 'speeds', y = 'result', yerr = 'error'):
			prefix = '%s-%s' % (bench, idir.rstrip('/'))
			pdf = get_output(prefix + '.pdf')
			eps = get_output(prefix + '.eps')
			txt = get_output(prefix + '.txt')
			graphs.append( (bench, d2c[idir], prefix) )
			kwds={ 'title':None, 'xlabel':'CPU Frequency [GHz]', 'ylabel':bench, 
				'ylim':ylims[bench],'xlim':xlims[idir] }
			save_series(series, pdf, **kwds)
			save_series(series, eps, **kwds)
			dump_series(series, txt)
			# now relative graphs
			series = relate_series(series, 'cpufreq')
			pdf = get_output('zrel_' + prefix + '.pdf')
			eps = get_output('zrel_' + prefix + '.eps')
			txt = get_output('zrel_' + prefix + '.txt')
			kwds={ 'title':None, 'xlabel':'CPU Frequency [GHz]', 'ylabel':'Ratio' }
			save_series(series, pdf, **kwds)
			save_series(series, eps, **kwds)
			dump_series(series, txt)
	latex_figures(get_output('figures.txt'), graphs)

