
/**
   Run N processes doing the same thing for some time.
   We generally want every process to run for 1 s.
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "common.h"

int main(int argc, char** argv) {
	int N = 5;
	lli i, loops;
	timestamp start, end;
	
	if (argc >= 2) {
		N = atoi(argv[1]);
	}
	
	loops = CALIBRATE(LOOP(), 1.);
	
	printf("# every process will perform loop %lld times.\n", loops);
	printf("# spawning %d processes.\n", N);
	
	int pids[N];
	
	start = get_time();
	for (i=0; i < N; i++) {
		pids[i] = fork();
		if (!pids[i]) { // child
			LOOPS(loops);
			exit(0);
		}
	}
	
	int retcode = 0;
	for (i=0; i < N; i++) {
		int status;
		if (pids[i] < 0) {
			fprintf(stderr, "fork() error\n");
			retcode = 1;
		} else {
			int ret = waitpid(pids[i], &status, 0);
			if (ret < 0 || !WIFEXITED(status) || WEXITSTATUS(status) != 0) {
				fprintf(stderr, "waitpid() error\n");
				retcode = 1;
			}
		}
	}
	end = get_time();
	
	timespan ts = get_diff(end, start);
	double cpu  = ts.cpu;
	double wall = ts.wall;
	
	printf("# wall: %.4lf, cpu: %.4lf\n", wall, cpu);
	printf("procs=%.8lf\n", loops / wall);
	
	return retcode;
}
