
/*
    This program is a frontend for all tests.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <libgen.h>
#include <errno.h>

typedef struct {
	char* name;
	char* path;
} test;

test tests[] = {
	{ "stressint", "stressint" },
	{ "stream", "stream" },
	{ "threads", "threads" },
	{ "sleeper", "sleeper" },
	{ "procs", "procs" },
	{ "udper", "udper" },
	{ "cpudper", "cpudper" },
	{ "linpack", "linpack" },
	{ 0, 0 },
};

int main(int argc, char** argv) {
	struct stat st;
	int owner;
	
	if (argc != 2) {
		fprintf(stderr, "Syntax: %s <test>\n", argv[0]);
		return 1;
	}
	
	if (getuid() != 0) { // non root user
		fprintf(stderr, "You must be a root to run this\n");
		return 1;
	}
	
	if (stat(argv[0], &st)) {
		fprintf(stderr, "stat() returned a non-zero value\n");
		return 1;
	}
	owner = st.st_uid;
	
	if (setuid(owner)) { /* run as a completely different user */
		fprintf(stderr, "setuid() returned a non-zero value\n");
		return 1;
	}
	
	char* dir = dirname(argv[0]);
	test* t = tests;
	while(t->name) {
		if (!strcmp(t->name, argv[1]))
			break;
		t++;
	}
	
	if (t->name) {
		char path[strlen(dir) + 1 + strlen(t->path) + 1];
		sprintf(path, "%s/%s", dir, t->path);
		execl(path, path, NULL);
		fprintf(stderr, "Error: %s (%s)\n", strerror(errno), path);
		return 1;
	} else {
		fprintf(stderr, "No such test: %s\n", argv[1]);
		return 1;
	}
}