
/**
 * This program: 
 *     (1) does some computation
 *     (2) measures the time of that computation
 *     (3) sleeps for the same amount of walltime
 *     (4) performs computation from (1) once again
 *     (5) prints out a total walltime of the runtime
 *
 * Remarks:
 *     - assuming no another workload, ratio (walltime / cputime) should be 1.5
 *     - generally: ratio = nr * 1.5, where nr is a number of CPU-bound tasks
 *       including this program
 **/

#include <stdio.h>
#include <unistd.h>
#include "common.h"

int main() {
	timestamp start, end;
	timespan ts;
	double wall, cpu;
	lli i, loops;
	
	loops = CALIBRATE(LOOP(), 1.);
	printf("# running %lld loops\n", loops);
	
	// printf("# phase 1\n");
	start = get_time();
	for (i=0; i < loops; i++)
		LOOP();
	end = get_time();
	ts = get_diff(end, start);
	cpu = ts.cpu;
	wall = ts.wall;
	
	// printf("# phase 2\n");
	if (usleep((long long int)(wall * 1000000.0)) != 0) {
		fprintf(stderr, "usleep() error\n");
		return 1;
	}

	// printf("# phase 3\n");
	for (i=0; i < loops; i++)
		LOOP();
	end = get_time();
	
	ts = get_diff(end, start);
	cpu  = ts.cpu; 
	wall = ts.wall;
	
	/* let's keep it proportional */
	printf("# wall: %.4lf, cpu: %.4lf, ratio: %.4lf\n", wall, cpu, wall / cpu);
	printf("sleeper=%.8lf\n", 2. * loops / wall);
	
	return 0;
}