
/**
   Run N threads doing the same thing for some time.
   We generally want every thread to run for 1 s.
 */

#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "common.h"

lli loops = 1;

void* thread(void* _) {
	LOOPS(loops);
	return (void*)0;
}

int main(int argc, char** argv) {
	int N = 5;
	int i;
	timestamp start, end;
	
	if (argc >= 2) {
		N = atoi(argv[1]);
	}
	
	loops = CALIBRATE(LOOP(), 1.);
	
	printf("# every process will perform loop %lld times.\n", loops);
	printf("# spawning %d threads.\n", N);
	
	pthread_t ids[N];
	int retcodes[N];
	
	start = get_time();
	for (i=0; i < N; i++) {
		retcodes[i] = pthread_create(&ids[i], NULL, thread, 0);
	}
	
	int retcode = 0;
	for (i=0; i < N; i++) {
		if (retcodes[i] != 0) {
			fprintf(stderr, "pthread_create() error\n");
			retcode = 1;
		} else {
			if (pthread_join(ids[i], NULL) != 0) {
				fprintf(stderr, "pthread_join() error\n");
				retcode = 1;
			}
		}
	}
	end = get_time();
	
	timespan ts = get_diff(end, start);
	double cpu  = ts.cpu;
	double wall = ts.wall;
	
	printf("# wall: %.4lf, cpu: %.4lf, ratio: %.4lf\n", wall, cpu, wall / cpu);
	printf("threads=%.8lf\n", loops / wall);
	return retcode;
}
